@@ -1229,7 +1229,7 @@ int powerof(UV n) {
   { UV cb = icbrt(n);  if (cb*cb*cb==n) return 3 * powerof(cb); }
   for (ib = 3; ib <= iblast; ib++) { /* prime exponents from 5 to 7-or-13 */
     UV k, pk, root, b = primes_small[ib];
-    root = (UV) ( pow(n, 1.0 / b ) + 0.01 );
+    root = (UV) ( pow(n, 1.0 / b ) + 1e-6 );
     pk = root * root * root * root * root;
     for (k = 5; k < b; k++)
       pk *= root;
@@ -1281,13 +1281,23 @@ int is_power(UV n, UV a)
       { UV cb = icbrt(n);
         return (cb*cb*cb != n)       ? 0 : (a == 3) ? 1 : is_power(cb, a/3); }
     if ((a % 5) == 0)
-      { UV r5 = (UV)(pow(n,0.2) + 0.0001);
+      { UV r5 = (UV)(pow(n,0.2) + 1e-6);
         return (r5*r5*r5*r5*r5 != n) ? 0 : (a == 5) ? 1 : is_power(r5, a/5); }
   }
   ret = powerof(n);
   if (a != 0) return !(ret % a);  /* Is the max power divisible by a? */
   return (ret == 1) ? 0 : ret;
 }
+UV rootof(UV n, UV k)
+{
+  switch (k) {
+    case 0:  return 0;
+    case 1:  return n;
+    case 2:  return isqrt(n);
+    case 3:  return icbrt(n);
+    default: return (UV) (pow((double)n, 1.0/(double)k)+1e-6);
+  }
+}
 
 UV valuation(UV n, UV k)
 {
@@ -1302,9 +1312,9 @@ UV valuation(UV n, UV k)
   return v;
 }
 
-UV popcount(UV n) {  return popcnt(n);  }
+UV mpu_popcount(UV n) {  return popcnt(n);  }
 
-UV popcount_string(const char* ptr, int len)
+UV mpu_popcount_string(const char* ptr, int len)
 {
   int i, *s, *sptr;
   UV count = 0;
